class WrappedIndicator:
    def __init__(self, to_next_impl, indicators):
        """

        :param to_next_impl: a function in the shape of (bar:Bar,indicator_values:dict)
                            and should return the calculated indicator value.
                            indicator_values is determined by what you passed in for the param `indicators`
        :param indicators: Base Indicators constructed by `BI` , which should be passed in dict
        """
        self.to_next_impl = to_next_impl
        self.indicators = indicators
        self.line = []

    def to_next(self, bar):
        indicator_values = {}
        for name, indicator in self.indicators.items():
            indicator_values[name] = indicator.to_next(bar)
            if indicator_values[name] is None:
                self.line.append(None)
                return None

        res = self.to_next_impl(bar, indicator_values)
        self.line.append(res)
        return res


class BaseIndicator:
    def __init__(self, impl, args, name=None):
        """

        :param impl: `impl` should be a standard Indicator instance,but not a class
        :param args: A list which means the data argument got from the bar
        """
        for arg in args:
            assert arg in ['ask_open', 'ask_close', 'ask_high', 'ask_low', 'bid_open', 'bid_close', 'bid_high',
                           'bid_low',
                           'volume'], "Args should in bar!"

        self.args = args
        self.impl = impl
        self.name = name

    def __repr__(self):
        return self.name

    @property
    def line(self):
        return self.impl.line

    @property
    def lines(self):
        return self.impl.lines

    def to_next(self, bar):
        data_list = []
        for arg in self.args:
            data_list.append(bar.__getattribute__(arg))
        return self.impl.to_next(*data_list)



from .ema import EMA


class MACD:
    def __init__(self, slow_period: int, fast_period: int):
        self.slow = EMA(slow_period)
        self.fast = EMA(fast_period)
        self.avg_diff = EMA(slow_period)
        self.line = []

    def to_next(self, sdata, fdata):
        _fast = self.fast.to_next(fdata)
        _slow = self.slow.to_next(sdata)
        if _fast is None or _slow is None:
            self.line.append(None)
            return None
        diff = _fast-_slow
        self.line.append(diff)
        _avg = self.avg_diff.to_next(diff)
        if _avg is None:
            self.line.append(None)
            return None
        res = _avg - 2*diff
        self.line.append(res)
        return res


#
# class SMACDNormalized:
#     def __init__(self, slow_period: int, fast_period: int, range_period: int):
#         self.smacd = SMACD(slow_period, fast_period)
#         self.range = Range(period=range_period)
#         self.line = []
#
#     def to_next(self, data):
#         _range = self.range.to_next(data, data)
#         _smacd = self.smacd.to_next(data)
#         res = None
#         if not (_range is None or _smacd is None):
#             res = _range - _smacd
#         self.line.append(res)
#         return res

export = MACD



import os
import glob
from setuptools import setup




try:
    from Cython.Distutils.extension import Extension
    from Cython.Distutils import build_ext as _build_ext
except ImportError as e:
    from setuptools import Extension
    from setuptools.command.build_ext import build_ext as _build_ext
    USING_CYTHON = False

else:
    USING_CYTHON = True

ext = 'pyx' if USING_CYTHON else 'c'
sources = []
cython_modules = ['indicator/indicators', 'quotation',
                  'trade', 'broker', 'runner', 'strategies']
package_data = {}
for m in cython_modules:
    package_data['newtrader.core.'+m.replace('/','.')] =  glob.glob(f'{m}/*.pxd')
    sources+=glob.glob(f'{m}/*.%s' % (ext,))
extensions = [
    Extension(source.split('.')[0].replace(os.path.sep, '.'),
              sources=[source],
            #   include_dirs=[numpy.get_include()]
              )
    for source in sources]

class build_ext(_build_ext):
    def finalize_options(self):
        _build_ext.finalize_options(self)
        # Prevent numpy from thinking it is still in its setup process:
        __builtins__.__NUMPY_SETUP__ = False
        import numpy
        self.include_dirs.append(numpy.get_include())

cmdclass = {'build_ext': build_ext}



setup(
    name="newtrader-core",
    version="0.0.13",
    author='Jiayi Su',
    author_email='nathaniel@finop.cloud',
    ext_modules=extensions,
    url="http://newtrader.finop.cloud",
    package_dir={
        "newtrader.core": ""
    },
    install_requires=[
        "numpy",
        "pandas",
        "prettytable",
        "tqdm",
        "tables",
        "click",
    ],
    setup_requires=[
        "numpy",
    ],
    package_data={
        "":["*.pxd"]
    },
    python_requires='>=3.8.1',
    cmdclass=cmdclass,
    packages=["newtrader.core",
              "newtrader.core.indicator",
              "newtrader.core.indicator.indicators",
              "newtrader.core.indicator.analyzer",
              "newtrader.core.indicator.generator",
              "newtrader.core.indicator.wrappers",
              "newtrader.core.broker",
              "newtrader.core.trade",
              "newtrader.core.quotation", "newtrader.core.strategies", "newtrader.core.runner"],
)



import numpy as np

supported = ['5min', '15min', '30min', '1h', '4h', '6h', '1D', '1W', '1M', '5D']

deltas = {

}

for s in supported:
    if s.endswith('min'):
        deltas[s] = np.timedelta64(int(s[:-3]), 'm')
    else:
        deltas[s] = np.timedelta64(int(s[: -1]), s[-1])



cimport numpy as np 





cdef class Context:
    cdef public str instrument
    cdef public object quotation
    cdef public object broker
    cdef public object parameters
    cdef public object parameters_instance
    cdef public np.float64_t initial_cash
    cdef public np.float64_t commission_rate
    cdef public str str_name
    cdef public str frequency
    cdef public object start_date
    cdef public object end_date



from quotation import LocalQuotation
from indicator import BaseIndicator
from algo import max_interval
import pandas as pd


def max_cumsum_interval(cumsum):
    max_i, min_i, max_up = max_interval.max_delta_interval(cumsum['profit'].to_list())
    return cumsum.index[min_i], cumsum.index[max_i], max_up


def analyze_instance(i: BaseIndicator, qt: LocalQuotation, trades: pd.DataFrame):
    """

    :param i: 技术指标实例
    :param qt:  行情数据
    :param trades: 交易列表
    :return:
    """
    for bar in qt.bar_p:
        i.to_next(bar)

    submit_table = trades.set_index('submit_datetime')

    indicator_df = pd.DataFrame(index=qt.df_p.index.copy().rename('submit_datetime'), data={i.name: i.line}).shift(
        periods=1)

    # 得到列联表
    # 这里是 把Indicator和订单表连接起来，索引是提交的时间
    joined_trade = submit_table.join(indicator_df)
    # 累计求和
    cumsum = joined_trade.groupby(i.name).sum().dropna(how='any').cumsum()

    start_v = 0.0
    end_v = 0.0

    if cumsum.shape[0] > 0:

        start_v, end_v, max_up = max_cumsum_interval(cumsum)

        filtered_trade = \
            joined_trade[(joined_trade[i.name] > start_v) & (joined_trade[i.name] < end_v)]
    else:
        filtered_trade = joined_trade

    filtered_trade[['exit_datetime', 'profit']].rename(columns={'exit_datetime': 'DateTime'}).set_index('DateTime').cumsum()
    exited = filtered_trade[['exit_datetime', 'profit']].rename(columns={'exit_datetime': 'DateTime'}).set_index(
        'DateTime').cumsum()
    submit = filtered_trade[['profit']]
    submit.index = submit.index.rename('DateTime')
    submit = submit.cumsum().shift(1).dropna()
    exited = exited.loc[exited.index.dropna()]
    result_pnl = submit.merge(exited, how='outer', on=['DateTime', 'profit'])
    result_pnl = result_pnl.sort_index()

    return i.name, start_v, end_v, result_pnl, cumsum, indicator_df



from xml.dom.minidom import parse
import xml.dom.minidom
import logging
from ..indicators import classes
from ..generator import BIG

logger = logging.getLogger(__name__)
counts = {

}


def parseXMLToIndicators(filename):
    DOMTree = xml.dom.minidom.parse(filename)
    indicators = DOMTree.getElementsByTagName('indicator')
    indicator_list = []

    for name in classes.keys():
        counts[name] = 0

    for indicator in indicators:
        label = indicator.getAttribute('label')
        class_name = indicator.getAttribute('class').upper()

        if label == '':
            label = 'None'

        if class_name not in classes.keys():
            logger.warning(f'invalid indicator class:{class_name} with label:{label},allowed:{classes.keys()}')
            continue

        if label == 'None':
            label = class_name + f'_{counts[class_name]}'
            counts[class_name] += 1

        iclass = classes[class_name]

        params_tag = indicator.getElementsByTagName('parameters')

        if len(params_tag) == 0:
            logger.warning(f'Indicator {label} has no parameters')
            continue

        params_tag = params_tag[0]
        params = params_tag.getElementsByTagName('parameter')
        plist = []
        for i, param_tag in enumerate(params):
            l = param_tag.childNodes[0].data.split(',')
            plist.append(l)

        new_plist = []
        if len(plist) > 0:
            for i, p_value in enumerate(plist[0]):
                v_list = []
                for p in plist:
                    v_list.append(eval(p[i]))
                new_plist.append(v_list)

        sources_list = []

        sources_tags = indicator.getElementsByTagName('sources')

        if len(sources_tags) == 0:
            logger.warning(f'Indicator {label} has no sources')

        for sources_tag in sources_tags:
            for source_tag in sources_tag.getElementsByTagName('source'):
                source = source_tag.childNodes[0].data.split(',')
                sources_list.append(source)

        i = BIG(iclass, new_plist, sources_list)
        indicator_list.extend(i)
    return indicator_list


